@@ -1,7 +1,8 @@
 import logging
-from typing import List, Tuple
+from typing import List, Tuple, Optional
 from pycparser import c_ast
-from pycparser.c_ast import Node, Assignment, If, While, For, Compound
+from pycparser.c_ast import Node, Assignment, If, While, For, Compound, \
+    ParamList
 
 from .relation_list import RelationList, Relation
 from .polynomial import Polynomial
@@ -18,7 +19,7 @@ class Analysis:
     @staticmethod
     def run(
             ast: c_ast, file_out: str = None, no_save: bool = False
-    ) -> Tuple[Relation, List[List[int]]]:
+    ) -> Tuple[Relation, List[List[int]], bool]:
         """Run MWP analysis on specified input file.
 
         Arguments:
@@ -27,14 +28,17 @@ class Analysis:
             no_save: Set true when analysis result should not be saved to file
 
         Returns:
-              Computed relation and list of non-infinity choices.
+              - Computed relation,
+              - list of non-infinity choices
+              - infinite/not infinite (boolean flag)
         """
 
-        choices = [0, 1, 2]
+        logger.debug("starting analysis")
 
-        function_body = ast.ext[0].body
+        choices = [0, 1, 2]
         index, combinations = 0, []
-        variables = Analysis.find_variables(function_body)
+        function_body, args = ast.ext[0].body, ast.ext[0].decl.type.args
+        variables = Analysis.find_variables(function_body, args)
         relations = RelationList.identity(variables=variables)
         total = len(function_body.block_items)
         delta_infty = False
@@ -53,29 +57,31 @@ class Analysis:
         if not delta_infty:
             combinations = relations.first.non_infinity(choices, index, dg)
 
+        # the evaluation is infinite when either of these conditions holds:
+        infinite = delta_infty or (relations.first.variables and index > 0 and
+                                   not combinations)
+
+        # save result to file unless explicitly disabled
         if not no_save:
             save_relation(file_out, relations.first, combinations)
 
+        # display results
         logger.debug(f'\nMATRIX{relations}')
 
-        if delta_infty:
-            logger.info('infinite')
-
-        # Should not raise here since delta_graph takes care of it
-        # conditional on: (1) that some variable(s) exist, so we do not
-        # conclude infinity on simple programs and
-        # (2) regarding index > see: issue #43
-        elif not combinations and relations.first.variables and index > 0:
-            logger.info('infinite undetected by delta graph')
-            assert False
+        if infinite:
+            logger.info('RESULT: infinite')
         else:
             logger.info(f'CHOICES:\n{combinations}')
 
-        return relations.first, combinations
+        # return results to caller
+        return relations.first, combinations, infinite
 
     @staticmethod
-    def find_variables(function_body: Compound) -> List[str]:
-        """Finds all local variable declarations in function body.
+    def find_variables(
+            function_body: Compound, param_list: Optional[ParamList]
+    ) -> List[str]:
+        """Finds all local variable declarations in function body and
+        parameter list.
 
         This method scans recursively AST nodes looking for
         variable declarations. For each declaration, the
@@ -83,7 +89,8 @@ class Analysis:
         a list of all discovered variable names.
 
         Arguments:
-            function_body: AST node with sub-nodes.
+            function_body: AST node with sub-nodes
+            param_list: AST function parameter list
 
         Returns:
             List of all discovered variable names, or
@@ -99,11 +106,16 @@ class Analysis:
                 for sub_node in node_.block_items:
                     recurse_nodes(sub_node)
 
-        # search for declarations
+        # search function body for local declarations
         if hasattr(function_body, 'block_items'):
             for node in function_body.block_items:
                 recurse_nodes(node)
 
+        # process param list which is a list of declarations
+        if param_list and hasattr(param_list, 'params'):
+            for node in param_list.params:
+                recurse_nodes(node)
+
         return variables
 
     @staticmethod
@@ -453,6 +465,7 @@ class Analysis:
             index: delta index
             node: AST node under analysis
             variables: list of unique variables in this operation where:
+
                 - variables[0][0] is the variable on left side of assignment
                 - variables[1][0] is left operand of binary operation
                 - variables[1][1] is right operand of binary operation
